#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: May 2 2008

## Given a locus, add the nearest feature in both strands

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $outFile, $verbose, $quiet);
my ($featureFile, $name, $iRef, $iId, $iStart, $iEnd, $iDir, $unique, $iExonStart, $iExonEnd, $iCdsStart, $iCdsEnd, $iInfo, $allow);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"i|input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"feature=s"		=> \$featureFile,
	"n|name=s"		=> \$name,
	"ref=i"			=> \$iRef,
	"id=i"			=> \$iId,
	"start=i"		=> \$iStart,
	"end=i"			=> \$iEnd,
	"dir=i"			=> \$iDir,
	"unique!"		=> \$unique,
	"es=i"			=> \$iExonStart,
	"ee=i"			=> \$iExonEnd,
	"cs=i"			=> \$iCdsStart,
	"ce=i"			=> \$iCdsEnd,
	"info=i"			=> \$iInfo,
	"allow=i"		=> \$allow,
) || die "\n";

checkOptions();

my %features;
my %ids;
my $break = 10000;

my ($tId, $tStart, $tEnd) = 0..2;
my $tDir = defined $iDir ? $tEnd+1 : $tEnd;
my $tExonStart = defined $iExonStart ? $tDir+1 : $tDir;
my $tExonEnd = defined $iExonEnd ? $tExonStart+1 : $tExonStart;
my $tCdsStart = defined $iCdsStart ? $tExonEnd+1 : $tExonEnd;
my $tCdsEnd = defined $iCdsEnd ? $tCdsStart+1 : $tCdsStart;
my $tInfo = defined $iInfo ? $tCdsEnd+1 : $tCdsEnd;

load($featureFile);
process($inFile);

#-------------------------------------------------------------------------------

sub getId
{
	my ($s) = @_;

	$s =~ s/ //g;
	return $unique ? $s : "${s}_".++$ids{$s};
}

sub load
{
	my ($fileName) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);

	while (<$in>)
	{
		next if /^#/ || /^\s*$/;
		my @a = split(/[\t\r\n]/);
		my $id = getId($a[$iId]);

		for (my $i = int(($a[$iStart]-$allow)/$break); $i <= int(($a[$iEnd]+$allow)/$break); $i++)
		{
			push(@{$features{$a[$iRef]}{$i}}, [ $id, @a[$iStart, $iEnd] ]);
			push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iDir]) if defined $iDir;
			push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iExonStart]) if defined $iExonStart;
			push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iExonEnd]) if defined $iExonEnd;
			if (defined $iCdsStart && defined $iCdsEnd && $a[$iCdsStart] != $a[$iCdsEnd])
			{
				push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iCdsStart]);
				push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iCdsEnd  ]);
			}
			push(@{$features{$a[$iRef]}{$i}[-1]}, $a[$iInfo]) if defined $iInfo;
			#print "$a[$iRef] $i\n";
		}

		$features{$a[$iRef]}{S} = int(($a[$iStart]-$allow)/$break) if !exists $features{$a[$iRef]}{S} || $features{$a[$iRef]}{S} > int(($a[$iStart]-$allow)/$break);
		$features{$a[$iRef]}{E} = int(($a[$iEnd  ]+$allow)/$break) if !exists $features{$a[$iRef]}{E} || $features{$a[$iRef]}{E} < int(($a[$iEnd  ]+$allow)/$break);
	}

	close($in) if defined $fileName;
}

sub process
{
	my ($fileName) = @_;

	print "Processing ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $out = openOutput($outFile);

	my $header = <$in>; $header =~ s/$/\t$name/;
	print $out $header;

	while (<$in>)
	{
		next if /^#/ || /^\s*$/;
		my @a = split(/[\t\r\n]/);
		my @fs = search(@a[0,1,2]);
		print $out join("\t", @a, join('|', @fs)), "\n";
	}

	close($in) if defined $fileName;
}

sub search
{
	my ($id, $start, $end) = @_;
	print "$id $start $end\n" if $verbose;

	my @backward = searchOneDirection($features{$id}, int($start/$break), $start, $end, -1);
	my @forward  = searchOneDirection($features{$id}, int($end  /$break), $start, $end,  1);
	print "backward: @backward\n" if @backward && $verbose;
	print "forward: @forward\n" if @forward && $verbose;
	return @backward ? @forward ? $backward[0] == $forward[0] ? merge(\@backward, \@forward)
	                            : $backward[0] < $forward[0] ? @backward : @forward
	                            : @backward : @forward ? @forward : (-1);
}

sub merge
{
	my ($rb, $rf) = @_;
	my %hash;

	foreach my $f (@$rb[1..$#$rb], @$rf[1..$#$rf])
	{
		$hash{$f}++;
	}

	return ($rb->[0], keys %hash);
}

sub searchOneDirection
{
	my ($hash, $index, $start, $end, $dir) = @_;
	my @ret = ();

	#die "Unknown S: $index, $start, $end\n" if !exists $hash->{S};
	#die "Unknown E: $index, $start, $end\n" if !exists $hash->{E};
	return if !exists $hash->{S};
	return if !exists $hash->{E};

	for (my $i = $index; $dir == 1 ? $i <= $hash->{E} : $i >= $hash->{S}; $i += $dir)
	{
		next if !exists $hash->{$i};

		foreach my $f (@{$hash->{$i}})
		{
			my ($d, %body);

			if ($dir == 1)
			{
				if ($f->[$tEnd] < $start)
				{
				}
				elsif ($f->[$tStart] > $end)
				{
					$d = $f->[$tStart] - $end;
				}
				else #if ($f->[$tEnd] <= $end || $f->[$tStart] <= $start)
				{
					$d = 0;
					searchExon($f, $start, $end, \%body) if defined $iExonStart && $iExonEnd;
				}
			}
			else
			{
				if ($f->[$tStart] > $end)
				{
				}
				elsif ($f->[$tEnd] < $start)
				{
					$d = $start - $f->[$tEnd];
				}
				else #if ($f->[$tEnd] >= $end || $f->[$tStart] >= $start)
				{
					$d = 0;
					searchExon($f, $start, $end, \%body) if defined $iExonStart && $iExonEnd;
				}
			}

			next if (!defined $d);
			if (!@ret || ($d < $ret[0] && $d >= $allow))
			{
				@ret = ();
				push(@ret, $d, featureStr($f, \%body));
			}
			elsif ($d == $ret[0] || $d <= $allow)
			{
				$ret[0] = $d if $ret[0] > $d;
				push(@ret, featureStr($f, \%body));
			}
		}

		last if @ret;
	}

	return @ret;
}

sub searchExon
{
	my ($f, $start, $end, $rbody) = @_;
	my @ss = split(/,/, $f->[$tExonStart]);
	my @es = split(/,/, $f->[$tExonEnd]);
	my @rs;
	my ($lutr, $rutr, $cds) = $f->[$tDir] eq '-' ? ("3'UTR", "5'UTR", "CDS") : ("5'UTR", "3'UTR", "CDS");
	print "CDS: $f->[$tCdsStart] $f->[$tCdsEnd]\n" if ($f->[$tCdsStart] && $f->[$tCdsEnd]) && $verbose;

	for (my $i = 0; $i < @ss; $i++)
	{
		if (isOverlap($start, $end, $ss[$i], $es[$i]))
		{
			if (!$f->[$tCdsStart] || !$f->[$tCdsEnd])
			{
			}
			elsif ($es[$i] < $f->[$tCdsStart])
			{
				push(@rs, $lutr);
			}
			elsif ($ss[$i] > $f->[$tCdsEnd  ])
			{
				push(@rs, $rutr);
			}
			elsif ($f->[$tCdsStart] <= $ss[$i] && $es[$i] <= $f->[$tCdsEnd]
				||  $f->[$tCdsStart] <= $start  && $end    <= $f->[$tCdsEnd])
			{
				push(@rs, $cds);
			}
			else
			{
				if ($ss[$i] < $f->[$tCdsStart])
				{
					if ($start <  $f->[$tCdsStart]) { push(@rs, $lutr); }
					if ($end   >= $f->[$tCdsStart]) { push(@rs, $cds ); }
				}
				if ($es[$i] > $f->[$tCdsEnd])
				{
					if ($start <= $f->[$tCdsEnd]) { push(@rs, $cds ); }
					if ($end   >  $f->[$tCdsEnd]) { push(@rs, $rutr); }
				}
			}
			push(@rs, 'exon');
			print " => @rs" if $verbose;
		}
		push(@rs, 'intron') if $i && isOverlap($start, $end, $es[$i-1]+1, $ss[$i]-1);
	}

	map {$rbody->{$_}++} @rs;
}

sub featureStr
{
	my ($f, $g) = @_;

	my $ret = "$f->[$tId]:$f->[$tStart]-$f->[$tEnd]";
	$ret .= $f->[$tDir] if defined $iDir;
	$ret .= ';' . join(',', map {"$_$g->{$_}"} sort keys %$g) if keys %$g;
	$ret .= '#' . $f->[$tInfo] if defined $iInfo;
	return $ret;
}

sub isOverlap 
{
	my ($l1, $r1, $l2, $r2) = @_;
	return ($l1 <= $l2 && $l2 <= $r1) || ($l1 <= $r2 && $r2 <= $r1) || ($l2 <= $l1 && $r1 <= $r2) ? 1 : 0;
}

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| zcat -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || (!$featureFile && !$inFile) || !$name || !$iRef || !$iId || !$iStart || !$iEnd)
	{
		die("Arguments: -name feature_name -ref index_ref -id index_id -start index_start -end index_end [-d index_dir] -f feature_file [-i in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	die "-es and -ee should be used together\n" if !$iExonStart ^ !$iExonEnd;
	if (!$iCdsStart ^ !$iCdsEnd) { die "-cs and -ce should be used together\n"; }
	elsif ($iCdsStart && $iCdsEnd && !$iDir) { die "-cs and -ce need -d\n"; }

	$iRef--; $iStart--; $iEnd--; $iId--;
	$iDir-- if $iDir;
	$iExonStart-- if $iExonStart;
	$iExonEnd-- if $iExonEnd;
	$iCdsStart-- if $iCdsStart;
	$iCdsEnd-- if $iCdsEnd;
	$iInfo-- if $iInfo;
	$allow = 0 if !$allow;
}
