#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: May 13 2008

## Description

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $

my ($out5, $end5, $body, $end3, $out3, $outside, $none) = ("5'out", "5'end", "body", "3'end", "3'out", "intergenic", "none");
my ($exon, $intron, $utr5, $cds, $utr3) = ('exon', 'intron', "5'UTR", 'CDS', "3'UTR");

use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $outFile, $verbose, $quiet, $field1, $field2, $scoreFlag, $binSize, $cdsFlag, $utrFlag, $allFlag,);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"f|field1=i"	=> \$field1,
	"g|field2=i"	=> \$field2,
	"score!"			=> \$scoreFlag,
	"bin=f"			=> \$binSize,
	"cds!"			=> \$cdsFlag,
	"utr!"			=> \$utrFlag,
	"all!"			=> \$allFlag,
) || die "\n";

checkOptions();

my (%class, %score);
my @endTypes = ("both_$end5", $end5, "both_$end3", $end3, $utr5, $utr3, $cds, $exon, $intron, $outside, $none);
my @utrTypes = ($utr5, $utr3, $cds, $exon, $intron, $end5, $end3, $outside, $none);
my @cdsTypes = ($cds, $utr5, $utr3, $exon, $intron, $end5, $end3, $outside, $none);
my @allTypes = ($end5, $end3, $utr5, $utr3, $cds, $exon, $intron, $outside, $none);
my (%subclass, %subfinal);
my $total = 0;

my ($oneKeyFt, $twoKeyFt, @combinedTypes) = $allFlag ? (\&oneAll, \&twoAll, @allTypes) : $cdsFlag ? (\&oneCdsFirst, \&twoCdsFirst, @cdsTypes) : $utrFlag ? (\&oneUtrFirst, \&twoUtrFirst, @utrTypes) : (\&oneEndFirst, \&twoEndFirst, @endTypes);
process($inFile);


#-------------------------------------------------------------------------------

sub getFinalType
{
	my ($str) = @_;
	$str = 'none' if !$str;
	my %types;

	foreach my $type (split(/,+/, $str))
	{
		$types{$type}++;
	}

	return $oneKeyFt->(\%types);
}

sub oneAll
{
	if (exists $_[0]->{$out5}) { delete $_[0]->{$out5}; $_[0]->{$outside}++; }
	if (exists $_[0]->{$out3}) { delete $_[0]->{$out3}; $_[0]->{$outside}++; }
	return $_[0];
}

sub oneEndFirst
{
	return exists $_[0]{$end5  } ? $end5   :
	       exists $_[0]{$end3  } ? $end3   :
	       exists $_[0]{$utr5  } ? $utr5   :
	       exists $_[0]{$utr3  } ? $utr3   :
	       exists $_[0]{$cds   } ? $cds    :
	       exists $_[0]{$exon  } ? $exon   :
	       exists $_[0]{$intron} ? $intron :
	       exists $_[0]{$none  } ? $none   : $outside
}	

sub oneCdsFirst
{
	return exists $_[0]{$cds   } ? $cds    :
	       exists $_[0]{$utr5  } ? $utr5   :
	       exists $_[0]{$utr3  } ? $utr3   :
	       exists $_[0]{$exon  } ? $exon   :
	       exists $_[0]{$intron} ? $intron :
	       exists $_[0]{$end5  } ? $end5   :
	       exists $_[0]{$end3  } ? $end3   :
	       exists $_[0]{$none  } ? $none   : $outside
}	

sub oneUtrFirst
{
	return exists $_[0]{$utr5  } ? $utr5   :
	       exists $_[0]{$utr3  } ? $utr3   :
	       exists $_[0]{$cds   } ? $cds    :
	       exists $_[0]{$exon  } ? $exon   :
	       exists $_[0]{$intron} ? $intron :
	       exists $_[0]{$end5  } ? $end5   :
	       exists $_[0]{$end3  } ? $end3   :
	       exists $_[0]{$none  } ? $none   : $outside
}	

sub twoAll
{
	my ($a, $b) = @_;
	my (%types, %dirs);
	map {if ($_ ne $outside) {$types{$_}++; $dirs{"$_+"}++;}} keys %$a;
	map {if ($_ ne $outside) {$types{$_}++; $dirs{"$_-"}++;}} keys %$b;
	if (!%types) { $types{$outside}++; $dirs{$outside}++; }
	return (\%types, \%dirs);
}

sub twoEndFirst
{
	my ($a, $b) = @_;

	return $b eq $end5   ? (0,'*') : (1,'+') if ($a eq $end5);
	return                           (1,'-') if ($b eq $end5);
	return $b eq $end3   ? (2,'*') : (3,'+') if ($a eq $end3);
	return                           (3,'-') if ($b eq $end3);
	return $b eq $utr5   ? (4,'*') : (4,'+') if $a eq $utr5;
	return                           (4,'-') if $b eq $utr5;
	return $b eq $utr3   ? (5,'*') : (5,'+') if $a eq $utr3;
	return                           (5,'-') if $b eq $utr3;
	return $b eq $cds    ? (6,'*') : (6,'+') if $a eq $cds;
	return                           (6,'-') if $b eq $cds;
	return $b eq $exon   ? (7,'*') : (7,'+') if $a eq $exon;
	return                           (7,'-') if $b eq $exon;
	return $b eq $intron ? (8,'*') : (8,'+') if $a eq $intron;
	return                           (8,'-') if $b eq $intron;
	return                           (9,' ');
}

sub twoCdsFirst
{
	my ($a, $b) = @_;

	return $b eq $cds    ? (0,'*') : (0,'+') if $a eq $cds;
	return                           (0,'-') if $b eq $cds;
	return $b eq $utr5   ? (1,'*') : (1,'+') if $a eq $utr5;
	return                           (1,'-') if $b eq $utr5;
	return $b eq $utr3   ? (2,'*') : (2,'+') if $a eq $utr3;
	return                           (2,'-') if $b eq $utr3;
	return $b eq $exon   ? (3,'*') : (3,'+') if $a eq $exon;
	return                           (3,'-') if $b eq $exon;
	return $b eq $intron ? (4,'*') : (4,'+') if $a eq $intron;
	return                           (4,'-') if $b eq $intron;
	return $b eq $end5   ? (5,'*') : (5,'+') if $a eq $end5;
	return                           (5,'-') if $b eq $end5;
	return $b eq $end3   ? (6,'*') : (6,'+') if $a eq $end3;
	return                           (6,'-') if $b eq $end3;
	return                           (7,'+');
}

sub twoUtrFirst
{
	my ($a, $b) = @_;

	return $b eq $utr5   ? (0,'*') : (0,'+') if $a eq $utr5;
	return                           (0,'-') if $b eq $utr5;
	return $b eq $utr3   ? (1,'*') : (1,'+') if $a eq $utr3;
	return                           (1,'-') if $b eq $utr3;
	return $b eq $cds    ? (2,'*') : (2,'+') if $a eq $cds;
	return                           (2,'-') if $b eq $cds;
	return $b eq $exon   ? (3,'*') : (3,'+') if $a eq $exon;
	return                           (3,'-') if $b eq $exon;
	return $b eq $intron ? (4,'*') : (4,'+') if $a eq $intron;
	return                           (4,'-') if $b eq $intron;
	return $b eq $end5   ? (5,'*') : (5,'+') if $a eq $end5;
	return                           (5,'-') if $b eq $end5;
	return $b eq $end3   ? (6,'*') : (6,'+') if $a eq $end3;
	return                           (6,'-') if $b eq $end3;
	return                           (7,' ');
}

sub process
{
	my ($fileName) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $out = openOutput($outFile);
	my $sum = openOutput("$outFile.stat");
	my $flag = 1;

	while (<$in>)
	{
		next if /^\s*$/;
		my @a = split(/[\t\r\n]/);
		my (%types1, %types2);

		if (!/^#/)
		{
			my $type1 = getFinalType($a[$field1]);
			my $type2 = getFinalType($a[$field2]);
			my ($ctype, $dir) = $twoKeyFt->($type1, $type2);
			my @keys = $allFlag ? sort keys %$ctype : ($combinedTypes[$ctype]);
			my @dirs = $allFlag ? sort keys %$dir   : ("$keys[0]$dir");

			for (my $i = 0; $i < @keys; $i++)
			{
				$class{$keys[$i]}++;
				$score{int($a[4]/$binSize)}{$keys[$i]}++ if $scoreFlag;
			}
			print $out join("\t", @a, join(',', @dirs)), "\n";
			$total++;
		}
		elsif ($flag)
		{
			my $h = $a[$field2]; $h =~ s/\(.*\)$/(Both)/;
			print $out join("\t", @a, $h), "\n";
			$flag = 0;
		}
		else
		{
			s/^#//;
			print $sum $_;
		}
	}

	close($in) if defined $fileName;

	foreach my $type (@combinedTypes)
	{
		print $sum join("\t", "both", $type, $class{$type} || 0), "\n";
	}

	if ($scoreFlag)
	{
		my @ss = sort {$a <=> $b} keys %score;
			print $sum join("\t", 'score', 'type', map {$_*$binSize} @ss), "\n";

		foreach my $type (@combinedTypes)
		{
			print $sum join("\t", 'score', $type);
			foreach my $s (@ss)
			{
				print $sum "\t", $score{$s}{$type} || 0;
			}
			print $sum "\n";
		}
	}

	print $sum "total\t$total\n";
}

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| zcat -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || !$field1 || !$field2)
	{
		die("Arguments: -f field_1 -g field_2 [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	$field1--; $field2--;
	$binSize = 1 if !defined $binSize;
}
