#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: May 2 2008

## Description

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $

my $field = 6;

use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $outFile, $verbose, $quiet);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"field=i"		=> \$field,
) || die "\n";

checkOptions();

$field--;
load($inFile);


#-------------------------------------------------------------------------------

sub load
{
	my ($fileName) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $out = openOutput($outFile);
	my @header;

	while (<$in>)
	{
		next if /^\s*$/;
		my @a = split(/[\t\r\n]/);

		if (!/^#/)
		{
			my (@min, @ret);

			foreach my $i ($field..$#a)
			{
				if ($a[$i] eq "-1")
				{
					push(@ret, 'inf');
				}
				else
				{
					my @b = split(/\|/, $a[$i]);
					my ($id, $fbeg, $fend, $fdir) = $b[1] =~ /(.+):(\d+)-(\d+)([+\-])*/;
					$b[0] = -$b[0] if !$fdir || $fdir eq '+' ? $a[2] < $fbeg : $a[1] > $fend;

					if (!@min || abs($min[0]) > abs($b[0]))
					{
						@min = ();
						push(@min, $b[0], $i);
					}
					elsif (abs($min[0]) == abs($b[0]))
					{
						push(@min, $i);
					}

					push(@ret, $b[0]);
				}
			}

			print $out join("\t", @ret, !@min ? ('inf', 'NA') : ($min[0], join(' ', @header[@min[1..$#min]]))), "\n";
		}
		elsif (!@header)
		{
			@header = @a;
			print $out '#', join("\t", @header[$field..$#header], 'Best'), "\n";
		}
	}

	close($in) if defined $fileName;
}


#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| zcat -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag)
	{
		die("Arguments: [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}
}
