#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: May 13 2008

## Description

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $

#my ($up5, $down5, $up3, $down3) = (1500, 500, 1500, 0);
my ($up5, $down5, $up3, $down3) = (2000, 500, 500, 500);
my ($out5, $end5, $body, $end3, $out3, $outside, $none) = ("5'out", "5'end", "body", "3'end", "3'out", "intergenic", "none");
my ($exon, $intron, $utr5, $cds, $utr3) = ('exon', 'intron', "5'UTR", 'CDS', "3'UTR");

use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $outFile, $geneFile, $verbose, $quiet, $field, $column, $bedIndex);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"gene=s"			=> \$geneFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"field=i"		=> \$field,
	"column=i"		=> \$column,
	"bed=i"			=> \$bedIndex,
	"up5=i"        => \$up5,
	"down5=i"      => \$down5,
	"up3=i"        => \$up3,
	"down3=i"      => \$down3,
) || die "\n";

checkOptions();

my (%class, %final);
my (%subclass, %subfinal);
my (%genes);

load($inFile);
process() if $geneFile;

#-------------------------------------------------------------------------------

sub load
{
	my ($fileName) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $out = openOutput($outFile);
	my $flag = 1;

	while (<$in>)
	{
		next if /^\s*$/;
		my @a = split(/[\t\r\n]/);
		my (%types, %subfeats, %detail);

		if (!/^#/)
		{
			my @b = split(/\|/, $a[$field]);

			if ($a[$field] eq "-1")
			{
				$types{$none}++;
			}
			else
			{
				my ($cbeg, $cend) = @a[1,2];
				for (my $i = 1; $i < @b; $i++)
				{
					my (%gtypes);
					my ($id, $gbeg, $gend, $dir) = $b[$i] =~ /(.+):(\d+)-(\d+)([+\-])/;
					$a[$field] = "-$a[$field]" if $b[0] && $i == 1 && $a[2] < $gbeg;
					my $extra;
					if (!$b[0] && $' =~ /;([^#]+)/)
					{
						map {s/\d+$//; $subfeats{$_}++;} split(/,/, $1);
						$extra = $1
					}
					my $glen = $gend - $gbeg + 1;
					my ($_down5, $_down3) = $glen - $down5 - $down3 > ($down5 >= $down3 ? $down5 : $down3) ? ($down5, $down3) : ($glen/3, $glen/3);

					# always gbeg is less than gend
					if ($dir eq '+')
					{
						if    ($cbeg >=  $gend+$up3)                                                         { $types{$out3}++; $gtypes{$out3}++; }
						elsif ($cend <=  $gbeg-$up5)                                                         { $types{$out5}++; $gtypes{$out5}++; }
						else
						{
							if (isOverlap($cbeg, $cend, $gbeg+$_down5, $gend-$_down3))                        { $types{$body}++; $gtypes{$body}++; }
							if ($up5 != -$down5 && isOverlap($cbeg, $cend, $gbeg-$up5+1,    $gbeg+$_down5-1)) { $types{$end5}++; $gtypes{$end5}++; }
							if ($up3 != -$down3 && isOverlap($cbeg, $cend, $gend-$_down3+1, $gend+$up3-1))    { $types{$end3}++; $gtypes{$end3}++; }
						}
					}
					else
					{
						if    ($cend <=  $gbeg-$up3)                                                         { $types{$out3}++; $gtypes{$out3}++; }
						elsif ($cbeg >=  $gend+$up5)                                                         { $types{$out5}++; $gtypes{$out5}++; }
						else
						{
							if (isOverlap($cbeg, $cend, $gbeg+$_down3, $gend-$_down5))                        { $types{$body}++; $gtypes{$body}++; }
							if ($up5 != -$down5 && isOverlap($cbeg, $cend, $gend-$_down5+1, $gend+$up5-1))    { $types{$end5}++; $gtypes{$end5}++; }
							if ($up3 != -$down3 && isOverlap($cbeg, $cend, $gbeg-$up3+1,    $gbeg+$_down3-1)) { $types{$end3}++; $gtypes{$end3}++; }
						}
					}

					addGene(\@a, $b[$i], \%gtypes) if $geneFile;
					if (exists $gtypes{$body})
					{
						die "$b[$i]\n" if (exists $gtypes{$out5} || exists $gtypes{$out3});
						delete $gtypes{$body};
					}
					$detail{$id} = join(',', sort keys %gtypes);
					if ($extra)
					{
						$detail{$id} .= "," if $detail{$id};
						$detail{$id} .= $extra;
					}
				}
			}

			if (exists $types{$body} || exists $types{$end5} || exists $types{$end3})
			{
				map {delete $detail{$_} if $detail{$_} =~ /out/} keys %detail;
				map {delete $types{$_} if /out/} keys %types;
			}

			foreach my $type (keys %types)
			{
				$class{$type}++;
			}

			$final{classPriority(\%types)}++; 

			#print $out join("\t", @a[0..$column], "@types");
			print $out join("\t", @a[0..$column], join('|', map {"$_($detail{$_})"} sort(keys(%detail))), join(',', sort keys %types));

			if (exists $types{$body} && keys %subfeats)
			{
				map {print $out ",$_"; $subclass{$_}++} sort keys %subfeats;
				$subfinal{bodyPriority(\%subfeats)}++;
			}

			print $out "\n";
		}
		elsif ($flag)
		{
			print $out join("\t", @a[0..$column], $a[$field], $a[$field]), "\n";
			$flag = 0;
		}
		else
		{
			print $out $_;
		}
	}

	close($in) if defined $fileName;

	print $out "#", $field+1, 'all';
	foreach my $type ($out5, $end5, $body, $end3, $out3, $none)
	{
		print $out "\t", $class{$type} || 0;
	}
	print $out "\n";

	print $out "#", $field+1, 'final';
	foreach my $type ($end5, $body, $end3, $outside, $none)
	{
		print $out "\t", $final{$type} || 0;
	}
	print $out "\n";

	print $out "#", $field+1, 'geneall';
	foreach my $type ($utr5, $cds, $utr3, $exon, $intron)
	{
		print $out "\t", $subclass{$type} || 0;
	}
	print $out "\n";

	print $out "#", $field+1, 'genefinal';
	foreach my $type ($utr5, $cds, $utr3, $exon, $intron)
	{
		print $out "\t", $subfinal{$type} || 0;
	}
	print $out "\n";
}

sub isOverlap 
{
	my ($x1, $x2, $y1, $y2, $allow) = @_;
	$allow = 0 if !defined $allow;

	($x1, $x2) = ($x2, $x1) if ($x1 > $x2);
	($y1, $y2) = ($y2, $y1) if ($y1 > $y2);
	return ($x1 <= $y1 && $y1 <= $x2-$allow) || ($x1+$allow <= $y2 && $y2 <= $x2) || ($y1 <= $x2 && $x2 <= $y2) ? 1 : 0;
}

sub classPriority
{
	return exists $_[0]{$end5} ? $end5 :
			 exists $_[0]{$end3} ? $end3 :
			 exists $_[0]{$body} ? $body :
			 exists $_[0]{$none} ? $none : $outside;
}

sub bodyPriority
{
	return exists $_[0]{$utr5} ? $utr5 :
			 exists $_[0]{$utr3} ? $utr3 :
			 exists $_[0]{$cds}  ? $cds  :
			 exists $_[0]{$exon} ? $exon : $intron;
}

sub addGene
{
	my ($rfeat, $hit, $rtypes) = @_;
	my $ft = classPriority($rtypes);
	return if $ft eq $none || $ft eq $outside || $ft eq $out5 || $ft eq $out3;

	my ($gid, $gbeg, $gend, $dir) = $hit =~ /(.+):(\d+)-(\d+)([+\-])/;
	$dir = '.' if !$dir;
	$genes{$gid}{'info'} = [$rfeat->[0], $gbeg, $gend, $dir] if !exists $genes{$gid};

	if ($ft eq $body && $' =~ /;([^#]+)/)
	{
		my %subfeats;
		map {s/\d+$//; $subfeats{$_}++} split(/,/, $1);
		$ft = bodyPriority(\%subfeats);
	}

	push(@{$genes{$gid}{$ft}}, join(',', join(':', "$rfeat->[1]-$rfeat->[2]", $bedIndex > 5 ? @$rfeat[3..5] : $bedIndex > 4 ? @$rfeat[3,4] : $bedIndex > 3 ? $rfeat->[3]: '')));
}

sub process
{
	my $out = openOutput($geneFile);
	print $out join("\t", '#ref', 'start', 'end', 'strand', 'ID', $end5, $utr5, $cds, $intron, $utr3, $end3), "\n";

	foreach my $gid (sort {
			$genes{$a}{'info'}[0] cmp $genes{$b}{'info'}[0] ||
			$genes{$a}{'info'}[1] <=> $genes{$b}{'info'}[1] ||
			$genes{$a}{'info'}[2] <=> $genes{$b}{'info'}[2]
		} keys %genes)
	{
		my @buf;
		print $out join("\t", @{$genes{$gid}{'info'}}, $gid);

		foreach my $t ($end5, $utr5, $cds, $intron, $utr3, $end3) 
		{
			push(@buf, exists $genes{$gid}{$t} ? join('|', @{$genes{$gid}{$t}}) : '-');
		}

		$buf[2] = join('|', @{$genes{$gid}{$exon}}) if $buf[1] eq '-' && $buf[2] eq '-' && $buf[4] eq '-' && exists $genes{$gid}{$exon};
		print $out "\t", join("\t", @buf), "\n";
	}
}

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| zcat -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || !$field || !$column || !$bedIndex)
	{
		die("Arguments: -c last_column_in_output -f field -b bed_index [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	$bedIndex--; $field--; $column--;
}
