#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: 2008

=description
Database: hg18    Primary Table: cpgIslandExt    Row Count: 28,226
Format description: Describes the CpG Islands (includes observed/expected ratio)
field	example	SQL type 	info 	description
chrom 	chr1	varchar(255) 	values 	Reference sequence chromosome or scaffold
chromStart 	18598	int(10) unsigned 	range 	Start position in chromosome
chromEnd 	19673	int(10) unsigned 	range 	End position in chromosome
name 	CpG: 116	varchar(255) 	values 	CpG Island
length 	1075	int(10) unsigned 	range 	Island Length
cpgNum 	116	int(10) unsigned 	range 	Number of CpGs in island
gcNum 	787	int(10) unsigned 	range 	Number of C and G in island
perCpg 	21.6	float 	range 	Percentage of island that is CpG
perGc 	73.2	float 	range 	Percentage of island that is C or G
obsExp 	0.83	float 	range 	Ratio of observed(cpgNum) to expected(numC*numG/length) CpG in island
=cut

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use Getopt::Long qw(:config no_ignore_case);
use File::Basename;

my ($helpFlag, $inFile, $featureFile, $sqlFile, $outFile, $verbose, $quiet, $version);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"feature=s"		=> \$featureFile,		## input file
	"output=s"		=> \$outFile,		## output file
#	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"version=s"		=> \$version,
	"sql=s"			=> \$sqlFile,
) || die "\n";

checkOptions();

$ENV{PATH} .= ':' . dirname($0);

my ($iref, $iid, $istart, $iend, $iinfo);

if    ($version =~ /hg19/    || $version =~ /mm9/   ) { ($iref, $iid, $istart, $iend, $iinfo) = (2, 5, 3, 4, 11); }
elsif ($version =~ /hg1[87]/ || $version =~ /mm[87]/) { ($iref, $iid, $istart, $iend, $iinfo) = (1, 4, 2, 3, 10); }
elsif ($sqlFile                                     ) { ($iref, $iid, $istart, $iend, $iinfo) = split(/\s+/, `goldenpath_sql.pl -a chrom -a name -a chromStart -a chromEnd -a obsExp $sqlFile`); }
else                                                  { die "Can't get indexes for the necessary attributes\n"; }

my $cmd = "addNearestNeighbor.pl -n CpgIsland -ref $iref -id $iid -start $istart -end $iend -nou -info $iinfo";
$cmd .= " -i $inFile" if $inFile;
$cmd .= " -f $featureFile" if $featureFile;
$cmd .= " -o $outFile" if $outFile;

#print STDERR "$cmd\n";
!system($cmd) || die "Error in $cmd\n";

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "zcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| zcat -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || (!$inFile && !$featureFile))
	{
		die("Arguments: -f goldenpath_knownGene [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	$version = '' if !$version;
}
