#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: 2010

## Description

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $inFile, $outFile, $verbose, $quiet, @attrs);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \$inFile,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"attribute=s"	=> \@attrs,
) || die "\n";

checkOptions();

my %attrs;
map { $attrs{$attrs[$_]} = $_ } 0..$#attrs;

my @ret;
load($inFile);

my $out = openOutput($outFile);
print $out join("\t", @ret), "\n";


#-------------------------------------------------------------------------------

sub load
{
	my ($fileName) = @_;

	my $in = openInput($fileName);
	my $i;

	while (<$in>)
	{
		next if /^#/ || /^\s*$/;
		if (/^\s*CREATE\s+TABLE/i)
		{
			$i = 1;
		}
		elsif ($i)
		{
			s/^\s+//;
			my @a = split;
			$a[0] =~ s/[`'"]//g;

			if (exists $attrs{$a[0]})
			{
				$ret[$attrs{$a[0]}] = $i;
				delete $attrs{$a[0]};
			}
			$i++
		}
	}

	die "Missing attributes: " . join(' ', keys(%attrs)) . "\n" if keys(%attrs);

	close($in) if defined $fileName;
}

#-------------------------------------------------------------------------------

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz(ip)?$/ ? "zcat $fileName |" : $fileName =~ /.bz(ip)?2$/ ? "bzcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| gzip -c > $fileName" : $fileName =~ /.bz(ip)?2$/ ? "| bzip2 -z -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	$inFile  = shift(@ARGV) if !defined $inFile  && @ARGV > 0;
	$outFile = shift(@ARGV) if !defined $outFile && @ARGV > 0;
	die "The output file name is the same as the input file name\n" if defined $inFile && defined $outFile && $inFile eq $outFile;

	if ($helpFlag || !@attrs)
	{
		die("Arguments: -a attribut [-a ...] [[-i] in_file] [[-o] out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}
}
