#!/usr/bin/perl -w
#!/usr/local/bin/perl -w
#!/bin/perl -w

## Author: Jeong-Hyeon Choi
## Date: Oct 14 2008

## join two or more files

## Usage:
## CHECK usage with option  -h

## $Id: perl_template,v 1.2 2007-09-21 22:51:23 jeochoi Exp $


use Getopt::Long qw(:config no_ignore_case);

my ($helpFlag, $outFile, $verbose, $quiet, $column, $ucsc, $refseq, $island, $rmsk, $rna, $snp);

GetOptions(
	"h|?|help"		=> \$helpFlag,	
	"input=s"		=> \@inFiles,		## input file
	"output=s"		=> \$outFile,		## output file
	"verbose+"		=> \$verbose,		## verbose output
	"quiet"			=> \$quiet,
	"ucsc=s"			=> \$ucsc,
	"refseq=s"		=> \$refseq,
	"cgi=s"			=> \$island,
	"rmsk=s"			=> \$rmsk,
	"rna=s"			=> \$rna,
	"snp=s"			=> \$snp,
	"column=i"		=> \$column
) || die "\n";

checkOptions();

my ($header, @sites);

=format
#Ref	Start	End	Name	Score	CpgIsland
chr1	467	469	11	12	-18129|CpG: 116_1:18598-19673#0.83	CpG: 116	5'out
#Ref	Start	End	Name	Score	RefGene(Top)	RefGene(Bot)	RefGene(Top)	RefGene(Bot)	RefGene(Both)
chr1	467	469	11	12	-58484|NM_001005484:58953-59871+	-610489|NM_001005224:610958-611897-|NM_001005277:610958-611897-|NM_001005221:610958-611897-	5'out	3'out	intergenic	OR4F5	OR4F16 OR4F29
#Ref	Start	End	Name	Score	Rmsk
chr1	467	469	11	12	0|Simple_repeat_1:0-468+|Satellite_1:468-1310-	Simple_repeat	body
#Ref	Start	End	Name	Score	RNA
chr1	467	469	11	12	-147177|snRNA_1:147646-147749-	snRNA	3'out
#Ref	Start	End	Name	Score	UcscGene(Top)	UcscGene(Bot)	UcscGene(Top)	UcscGene(Bot)	UcscGene(Both)
chr1	467	469	11	12	-1267|uc001aaa.1:1736-4121+	-4089|uc001aab.1:4558-14764-|uc001aac.1:4558-19346-|uc001aad.1:4558-7231-|uc001aae.1:4558-9622-	5'end	3'out	5'end	BC032353	FLJ00038 DKFZp434K1323
=cut

my @fields = 0..$column-1;
load($ucsc  , [$column+5,$column+6,$column+4], ['UCSC(Top)', 'UCSC(Bot)', 'UCSC(class)']) if $ucsc;
load($refseq, [$column+5,$column+6,$column+4], ['RefSeq(Top)', 'RefSeq(Bot)', 'RefSeq(class)']) if $refseq;
load($island, [$column+1,$column+2], ['CGI', 'CGI(class)']) if $island;
load($rmsk,   [$column+1,$column+2], ['Repeat', 'Repeat(class)']) if $rmsk;
load($rna,    [$column+1,$column+2], ['RNA', 'RNA(class)']) if $rna;
load($snp,    [$column+1,$column+2], ['SNP', 'SNP(class)']) if $snp;

output();

#-------------------------------------------------------------------------------

sub load
{
	my ($fileName, $rindexes, $rnames) = @_;

	print "Loading ", $fileName || 'STDIN', " ...\n" if !$quiet;
	my $in = openInput($fileName);
	my $flag = 1;
	my $i = 0;

	while (<$in>)
	{
		my @a = split(/[\t\r\n]/);

		if ($flag)
		{
			$header = join("\t", @a[@fields]) if !$header;
			$header .= join("\t", '', @$rnames);
			$flag = 0;
			next;
		}

		next if /^#/ || /^\s*$/;
		$sites[$i  ]  = join("\t", @a[@fields]) if !$sites[$i];
		$sites[$i++] .= join("\t", '', map {$_||''} @a[@$rindexes]);
	}

	close($in) if defined $fileName;
}

sub output
{
	my $out = openOutput($outFile);

	print $out $header, "\n";

	foreach my $key (@sites)
	{
		print $out $key, "\n";
	}
}

#-------------------------------------------------------------------------------

sub _getDistinctName
{
	my ($rfiles, $del) = @_;
	$del = '[\._]' if !$del;

	my @arrays;
	map { my @a = split(/$del/, fileparse($_)); push(@arrays, \@a) } @$rfiles;

	my %strs;
	for (my $i = 0; $i < @arrays; $i++)
	{
		map { $strs{$_}{$i}++ } @{$arrays[$i]};
	}

	foreach my $s (keys %strs)
	{
		next if keys(%{$strs{$s}}) < @arrays;
		for (my $i = 0; $i < @arrays; $i++)
		{
			for (my $j = 0; $j < @{$arrays[$i]}; $j++)
			{
				if ($arrays[$i][$j] eq $s) { $arrays[$i][$j] = ''; last; }
			}
		}
	}

	return map {join('', @$_)} @arrays;
}

sub _parseRangeArg1
{
	my @array = _parseRangeArg(@_);
	_decrease(\@array, 1);
	return @array;
}

sub _decrease
{
	my ($array, $value) = @_;
	map {$_-=$value} @$array;
}

sub _parseRangeArg
{
	my %ret;

	foreach my $arg (@_)
	{
		my @a;

		foreach my $r (split(/,/, $arg))
		{
			push(@a, ($r =~ /(\d+)-(\d+)/) ? $1..$2 : $r);
		}

		map {$ret{$_}++} @a;
	}

	return sort {$a<=>$b} keys %ret;
}

sub openInput
{
	my ($fileName) = @_;

	return *STDIN unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz(ip)?$/ ? "zcat $fileName |" : $fileName =~ /.bz(ip)?2$/ ? "bzcat $fileName |" : $fileName) || die("Open error: $fileName");
	return $fd;
}

sub openOutput
{
	my ($fileName) = @_;

	return *STDOUT unless defined $fileName;

	my ($fd);
	open($fd, $fileName =~ /.gz$/ ? "| gzip -c > $fileName" : $fileName =~ /.bz(ip)?2$/ ? "| bzip2 -z -c > $fileName" : ">$fileName") || die("Open error: $fileName");
	return $fd;
}

sub checkOptions
{
	push(@inFiles, @ARGV) if !@inFiles  && @ARGV > 0;

	if ($helpFlag || !$column || (!$ucsc && !$refseq && !$island && !$rna && !$rmsk && !$snp))
	{
		die("Arguments: -c column [-ucsc ucsc_class] [-ref ref_class] [-cpg cpgisland_class] [-rmsk repeast_class] [-rna rna_class] [-snp snp_class] [-o out_file] [-v] [-q]\n"
		  . "\t\n"
		  );
	}

	$column--;
}
